/* .MiK. SoftWare

  Multilingual Terminal Emulator "MTR".

 Copyright (C) 2000 Mike I. Kozin. ALL RIGHT RESERVED
 
 On Base "MuTerminal"
 Copyright (C) 1998,99 Kazuho Okui and Takashi Murai. ALL RIGHT RESERVED
  
 This file is part of MTR.

 MTR is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 TermBuff.h,v 0.1 26-Sep-2000
***************************************************************************/

#ifndef TERMBUFFER_H_INCLUDED
#define TERMBUFFER_H_INCLUDED

#include <SupportDefs.h>
#include "TermConst.h"
#include "CurPos.h"

// Scroll direction
const int array_up = 0;
const int array_down = 1;

#define ARRAY_SIZE 512

#define MIN_COLS 10
#define MAX_COLS 256

#define ROW(x) (((x) + mRowOffset) % buffer_size)

typedef struct term_buffer
{
  uchar code[4];
  int16 attr;
  int16 status;
} term_buffer;

class CurPos;
class BString;

class TermBuffer
{
public:
  TermBuffer(int row, int col);
  ~TermBuffer();

  //
  // Get and Put charactor.
  //
  int	GetChar (int row, int col, uchar *u, ushort *attr);

  int	GetString (int row, int col, int num, uchar *u, ushort *attr);
  void  GetStringFromRegion (BString &copyStr);  

  void	WriteChar (const CurPos &pos, const uchar *u, ushort attr);
  void	WriteCR (const CurPos &pos);
  void	InsertSpace (const CurPos &pos, int num);

  //
  // Delete Charactor.
  //
  void	DeleteChar (const CurPos &pos, int num);
  void	EraseBelow (const CurPos &pos);

  //
  // Movement and Scroll buffer.
  //
  void	ScrollRegion (int top, int bot, int dir, int num);
  void	ScrollLine (void);
	
  //
  // Resize buffer.
  //
  void	ResizeTo(int newRows, int newCols, int offset);

  //
  // Clear contents of TermBuffer.
  //
  void	ClearAll (void);

  // Set / Unset Selection
  void	Select (const CurPos &start, const CurPos &end);
  void	DeSelect (void);

  int	CheckSelectedRegion (const CurPos &pos);
  bool	FindWord (const CurPos &pos, CurPos *start, CurPos *end);

  void	GetCharFromRegion (int x, int y, BString &str);
  void	AvoidWaste (BString &str);

  int32 GetArraySize (void);
  
  /*
   * PRIVATE MEMBER FUNCTIONS.
   */
private:

  //
  // Erase 1 column.
  //
  void	EraseLine (int row);

  /*
   * DATA MEMBER.
   */
  term_buffer 	**mBase;
  
  int		mColSize;
  int		mRowSize;
  int		mNowColSize;
  int		mRowOffset;

  CurPos	mSelStart;
  CurPos	mSelEnd;

  int		buffer_size;
	
};

#endif //TERMARAY_H_INCLUDED
